/**
 * Pyx4me framework
 * Copyright (C) 2005-2007 Michael Lifshits
 * Copyright (C) 2005-2007 Vlad Skarzhevskyy
 *
 * @version $Id: ReflectionTestCaseHelper.java 1 2007-03-09 04:03:48Z vlads $
 */
package cldcunit.runner;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Vector;

import cldcunit.runner.BaseTestCaseHelper;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestMethod;

public class ReflectionTestCaseHelper extends BaseTestCaseHelper {

	public static final String SUITE_METHODNAME= "suite";

	public ReflectionTestCaseHelper() {
	}

	 public TestMethod[] createAllTestMethods() {
		 Vector testMethodsVector = new Vector();
			try {
				getTestConstructor(getTestCase().getClass()); // Avoid generating multiple error messages
			} catch (NoSuchMethodException e) {
				testMethodsVector.addElement(createWarningTestMethod("Class "+getTestCase().getName()+" has no public constructor TestCase(String name) or TestCase()"));
				return (TestMethod[])testMethodsVector.toArray(new TestMethod[0]);
			}

			if (!Modifier.isPublic(getTestCase().getClass().getModifiers())) {
				testMethodsVector.addElement(createWarningTestMethod("Class "+getTestCase().getName()+" is not public"));
				return (TestMethod[])testMethodsVector.toArray(new TestMethod[0]);
			}

			Class superClass= getTestCase().getClass();
			while (Test.class.isAssignableFrom(superClass)) {
				Method[] methods= superClass.getDeclaredMethods();
				for (int i= 0; i < methods.length; i++) {
						if (isTestMethod(methods[i])) {
							if (! isPublicTestMethod(methods[i])) {
								testMethodsVector.addElement(createWarningTestMethod("Test method isn't public: "+methods[i].getName()));
								return (TestMethod[])testMethodsVector.toArray(new TestMethod[0]);
							} else {
								testMethodsVector.addElement(createTestMethod(methods[i].getName()));
							}
						}
				}
				superClass= superClass.getSuperclass();
			}
			if (testMethodsVector.size() == 0) {
				testMethodsVector.addElement(createWarningTestMethod("No tests found in "+getTestCase().getClass().getName()));
				return (TestMethod[])testMethodsVector.toArray(new TestMethod[0]);
			}
			return (TestMethod[])testMethodsVector.toArray(new TestMethod[0]);
		}
	 
		/**
		 * Gets a constructor which takes a single String as
		 * its argument or a no arg constructor.
		 */
		public static Constructor getTestConstructor(Class theClass) throws NoSuchMethodException {
			Class[] args= { String.class };
			try {
				return theClass.getConstructor(args);	
			} catch (NoSuchMethodException e) {
				// fall through
			}
			return theClass.getConstructor(new Class[0]);
		}

		private boolean isPublicTestMethod(Method m) {
			return isTestMethod(m) && Modifier.isPublic(m.getModifiers());
		 }
		 
		private boolean isTestMethod(Method m) {
			String name= m.getName();
			Class[] parameters= m.getParameterTypes();
			Class returnType= m.getReturnType();
			return parameters.length == 0 && name.startsWith("test") && returnType.equals(Void.TYPE);
		 }

		protected void runTest(String methodName) throws Throwable {
			Method runMethod= null;
			try {
				// use getMethod to get all public inherited
				// methods. getDeclaredMethods returns all
				// methods of this class but excludes the
				// inherited ones.
				runMethod = getTestCase().getClass().getMethod(methodName, null);
			} catch (NoSuchMethodException e) {
				TestCase.fail("Method \""+methodName+"\" not found");
			}
			if (!Modifier.isPublic(runMethod.getModifiers())) {
				TestCase.fail("Method \""+methodName+"\" should be public");
			}

			try {
				runMethod.invoke(getTestCase(), new Class[0]);
			}
			catch (InvocationTargetException e) {
				e.fillInStackTrace();
				throw e.getTargetException();
			}
			catch (IllegalAccessException e) {
				e.fillInStackTrace();
				throw e;
			}
		}

		 public TestMethod createTestMethod(String name) {
				return new TestMethod(name) {
					public void run(junit.framework.TestCase testCase) throws Throwable {
						runTest(this.getName());
					}
				};
		 }
		 
		 public TestMethod createWarningTestMethod(final String message) {
				return new TestMethod("warning") {
					public void run(junit.framework.TestCase testCase) throws Throwable {
						TestCase.fail(message);
					}
				};
		 }

		public Test getSuite() {
			Method suiteMethod= null;
			try {
				suiteMethod= getTestCase().getClass().getMethod(SUITE_METHODNAME, new Class[0]);
		 	} catch(Exception e) {
				return null;
			}
			if (! Modifier.isStatic(suiteMethod.getModifiers())) {
				TestCase.fail("Suite() method must be static");
				return null;
			}
			Test test= null;
			try {
				test= (Test)suiteMethod.invoke(null, new Class[0]); // static method
				if (test == null)
					return test;
			}
			catch (InvocationTargetException e) {
				TestCase.fail("Failed to invoke suite():" + e.getTargetException().toString());
				return null;
			}
			catch (IllegalAccessException e) {
				TestCase.fail("Failed to invoke suite():" + e.toString());
				return null;
			}

			return test;
		}
}
